local icons = {}
local classes = {}
local f = CreateFrame("Frame")
local frames_to_update = {}
local right_click_fixed = {}
local in_bg

for classID=1,MAX_CLASSES do
	local _,classTag = GetClassInfoByID(classID)
	local numTabs = GetNumSpecializationsForClassID(classID)
	classes[classTag] = {}
	for i=1,numTabs do
		local _,spec_name,_,spec_icon = GetSpecializationInfoForClassID(classID,i)
		classes[classTag][spec_name] = spec_icon
	end
end

hooksecurefunc("CompactUnitFrame_UpdateInRange",function(frame)
	local inRange,checkedRange = UnitInRange(frame.displayedUnit)
	if checkedRange and not inRange then
		if in_bg then
			frame:SetAlpha(0.25)
		else
			frame:SetAlpha(1)
		end
	end
end)

hooksecurefunc("CompactUnitFrame_UpdateName",function(frame)
	frame.name:SetText(GetUnitName(frame.unit,true):match("[^-]+"))
	local _,class = UnitClass(frame.unit)
	local color = RAID_CLASS_COLORS[class]
	if color then
		frame.name:SetTextColor(color.r,color.g,color.b)
		if in_bg then
			frames_to_update[frame] = true
			f:Show()
		end
		frame.name:Show()
	end
	if not right_click_fixed[frame] and not InCombatLockdown() then
		frame:RegisterForClicks("AnyDown")
		frame:SetAttribute("type2","macro")
		frame:SetAttribute("macrotext2","/run MouselookStart()")
		right_click_fixed[frame] = true
	end
end)

hooksecurefunc("CompactUnitFrame_UpdateHealthColor",function(frame)
	local r,g,b
	if UnitIsConnected(frame.unit) then
		if UnitIsFriend("player",frame.unit) then
			r,g,b = 0,1,0
		else
			r,g,b = 1,0,0
		end
	end
	if r~=frame.healthBar.r or g~=frame.healthBar.g or b~=frame.healthBar.b then
		frame.healthBar:SetStatusBarColor(r,g,b)
		frame.healthBar.r,frame.healthBar.g,frame.healthBar.b = r,g,b
	end
end)

f:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_ENTERING_WORLD" then
		local _,InstanceType = IsInInstance()
		in_bg = InstanceType=="pvp"
		if in_bg then
			f:RegisterEvent("GROUP_ROSTER_UPDATE")
			f:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
		else
			f:UnregisterEvent("GROUP_ROSTER_UPDATE")
			f:UnregisterEvent("PLAYER_SPECIALIZATION_CHANGED")
		end
	elseif in_bg then
		f:Show()
	end
	if event=="GROUP_ROSTER_UPDATE" and not InCombatLockdown() then
		for k,v in pairs(frames_to_update) do
			if not k:IsShown() then
				frames_to_update[k] = nil
			end
		end
	end
end)
f:RegisterEvent("PLAYER_ENTERING_WORLD")

f:SetScript("OnUpdate",function(self,elapsed)
	self.timer1 = self.timer1 and self.timer1+elapsed or 0
	if self.timer1>0.25 then
		self.timer2 = self.timer2 and self.timer2+self.timer1 or 0
		self.timer1 = 0
		if self.timer2>5 then
			self.timer2 = 0
			f:Hide()
		end
		RequestBattlefieldScoreData()
		for index=1,GetNumBattlefieldScores() do
			local name,_,_,_,_,faction,_,_,classToken,_,_,_,_,_,_,talentSpec = GetBattlefieldScore(index)
			if classToken then
				icons[name] = classes[classToken][talentSpec]
			end
		end
		for k,v in pairs(frames_to_update) do
			if not k:IsShown() or not k.unit then
				frames_to_update[k] = nil
			else
				local texture = icons[GetUnitName(k.unit,true)]
				if texture then
					k.roleIcon:SetTexture(texture)
					k.roleIcon:SetTexCoord(0,1,0,1)
					k.roleIcon:SetSize(16,16)
					k.roleIcon:Show()
				end
			end
		end
	end
end)